<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Acidblur Effect</H3>
            <p>Author: Gareth Noyce (<a href="http://www.korruptor.demon.co.uk/">www.korruptor.demon.co.uk</a><br>
            Submission date: September 13, 2002</p>
	  
            <p>Description: An 'Acidblur' effect using pygame.transform and Numeric.</p>
            
            <p align="center"><img src="acidblur.jpg" width="320" height="200"></p>

            <p>Download: <a href="acidblur.zip">acidblur.zip</a></p>	  

            <p>pygame version required: Any with Surfarray<br>
            SDL version required: Any<br>
            Python version required: Any with Numeric</p>

            <p>Comments: Acidblur is another aggressively-commented effects script from Gareth Noyce. This is a pretty well-known effect, and is produced by
            a combination of blurring and zooming transforms. Mr. Noyce has thoughtfully included a sample graphic of pygame mascot Monty, so you can see the
            effect in action.</p>
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
Program		:	Acid Blur (not that I'd know)
Author		:	Gareth "Korruptor" Noyce (<a href=http://www.korruptor.demon.co.uk/>Home</a>)
Depend		:	Numeric, Pygame with Surfarray
Description	:

    This is probably familiar to anyone who uses iTunes or Winamp visualisations, 
and it's another *really* easy one.

Basically, a four step process:

1) Scale the display surface
2) Blur the scaled surface
3) Draw something onto the blurred, scaled surface
3) Blit the central portion of this to the display

I'll leave out the goto 1 ;-)

The blur is a basic convolution, as demonstrated by Pete in his excellent Surfarray
tutorial. The only thing here that's new is our use of pygame's transform functions 
to scale the surface.

This is dedicated to slm, who seems to think I'm writing a visualisation library. :)
"""</span>

import pygame, pygame.transform, pygame.image
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pycomment"># Glob decs</span>

<span class="pycomment"># Screen resolution...</span>
RES 	= array((320,200))
PI 	= 3.14159
DEG2RAD = PI/180

<span class="pycomment"># -----------------------------------------------------------------------------------</span>
<span class="pydef">def main():</span>
    
    pygame.init()

    <span class="pycomment"># Setup an 8bit screen...</span>
    screen = pygame.display.set_mode(RES,0,8)
    
    <span class="pycomment"># Load a sprite and setup the palettes and colorkey info</span>
    sprite = pygame.image.load("<span class="pystring">sprite.gif</span>")
    sprite.set_palette(sprite.get_palette())
    sprite.set_colorkey((0,0,0))

    screen.set_palette(sprite.get_palette())

    <span class="pycomment"># This is our "working surface"...</span>
    worksurf = pygame.Surface(RES,0,8)
    worksurf.set_palette(sprite.get_palette())

    
    <span class="pycomment"># Angle deltas for sin/cos</span>
    xd = 0
    yd = 0
    
    <span class="pycomment"># Fruity loops...</span>
    while 1:
        <span class="pycomment"># Have we received an event to quit the program?</span>
        for e in pygame.event.get():
            if e.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
                return

        <span class="pycomment"># Calculate a new x/y position for the sprite...</span>
        x    = ((RES[1]/4))*cos((xd*DEG2RAD) * 2.75)
        y    = ((RES[1]/4))*sin((yd*DEG2RAD) * 1.13)

        <span class="pycomment"># Increment the 'angles'</span>
        xd  += 1.5
        yd  += 3

        <span class="pycomment"># Scale the working surface up by 16px x 16px -- obviously</span>
        <span class="pycomment"># smaller values == a slower blur.</span>
        worksurf = pygame.transform.scale(screen, (RES[0]+16,RES[1]+16))

        <span class="pycomment"># Copy the surf array of the scaled surface to a temporary array</span>
        tmp = pygame.surfarray.array2d(worksurf)
        
        <span class="pycomment"># Workarr will contain the 'blurred' version of the scaled surface</span>
        workarr = array(tmp)

        <span class="pycomment"># This is exactly the same blur that's demonstrated in the Surfarray
        # tutorial by Pete Shinners. An 8px version would probably look a 
        # little better, but it's no great shakes for this demo...</span>
        workarr[1:,:]  += tmp[:-1,:]*8
        workarr[:-1,:] += tmp[1:,:]*8
        workarr[:,1:]  += tmp[:,:-1]*8
        workarr[:,:-1] += tmp[:,1:]*8
        workarr /= 33
        
        <span class="pycomment"># Take the central portion of the scaled (and now blurred) array
        # and blit it to the screen. This creates an inward zoom...
        # If you change the blit offset, you can move the blur 
        # e.g.: workarr[16:RES[0]+16,8:RES[1]+8].shape == left motion,
        # so moving them about a sine would be easy and look quite nice ;)</span>
        blit_array(screen, workarr[8:RES[0]+8,8:RES[1]+8])
        screen.blit(sprite,(x+110,y+50))
        
        <span class="pycomment"># show the results...</span>
	pygame.display.update()

if __name__ == '<span class="pystring">__main__</span>': main()

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

