<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Box Selection Tool</H3>
            <p>Author: Frank Raiser (crashchaos at gmx.net)<br>
            Submission date: August 11, 2001</p>
	  
            <p>Description: Manages mouse handling for box selection with rubberbanded box drawing.</p>

            <p>Download: <a href="selection.py.gz">selection.py.gz</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: 2.0</p>

            <p>Comments: A handy module for managing box selection. The module handles the mouse events, and returns the area selected by the mouse when the mouse is released. It also draws an rubberband style outline of the area selected. The module includes a simple test program, running the module allows you to test it out.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
selection.py - C&C like selection rectangles
Raiser, Frank - Aug 3, 2k++
crashchaos at gmx.net

This is a simple and ready-to-use class which adds C&C (Command & Conquer)
like selection rectangles to your games/apps so that the user can resize
a rectangle to cover some area on the screen (f.ex. to choose all units in
that area)

You can run this sample right away using the supplied main() method if you
want to check the effect.

A short description on the usage of this class:
1) Initiate the class passing it the surface to display the rectangle on,
   the starting point of your selection rectangle and an optional color
   in which to draw the rectangle (defaults to black)
2) Check for MOUSEMOTION and then call updateRect() and draw() methods (this
   is doing the main work)
3) After the user released the mousebutton (or whenever you want actually)
   make a final call to updateRect() (it will return the selection rectangle
   and you might want to save that in some var ;).
4) Don't forget to call the hide() method to clear the final selection
   rectangle (unless you're totally redrawing the screen anyways)

Possible customizations:
1) the default drawing color (line X)
2) use another surfarray method (pixels3d is used here which only works
   for surfaces with 32 or 24 bpp - shouldn't be that hard)
3) something else I didn't think of :)

Dependencies:
1) pygame w/ surfarray support
2) PyNumeric
"""</span>

import pygame,pygame.surfarray
from pygame.locals import *
import Numeric,copy

<span class="pydef">class SelectionRect:</span>
    <span class="pycomment">""" class SelectionRect utility class for using selection rectangles"""</span>
    <span class="pydef">def __init__(self,screen,start,col=(0,0,0)):</span>
        <span class="pycomment">""" __init__(self,screen,start,col=(0,0,0))
        Constructor. Pass starting point of selection rectangle in 'start'
        and color value in which the selection rectangle shall be drawn
        in 'col'
        """</span>
        self.start         = start
        self.col           = col
        self.oldrect       = start[0],start[1],1,1
        tmp                = screen.get_at((start[0],start[1]))[:3]
        self.screen_backup = [[tmp],[tmp],[tmp],[tmp]]
        
    <span class="pydef">def updateRect(self,now):</span>
        <span class="pycomment">""" updateRect(self,now) -> rect tuple
        This returns a rectstyle tuple describing the selection rectangle
        between the starting point (passed to __init__) and the 'now' edge and
        updates the internal rectangle information for correct drawing.
        """</span>
        x,y = self.start
        mx,my = now
        if mx < x:
            if my < y:
                self.rect = mx,my,x-mx,y-my
            else:
                self.rect = mx,y,x-mx,my-y
        elif my < y:
            self.rect = x,my,mx-x,y-my
        else:
            self.rect = x,y,mx-x,my-y
        return self.rect

    <span class="pydef">def draw(self,screen):</span>
        <span class="pycomment">""" draw(self,screen)
        This hides the old selection rectangle and draws the current one
        """</span>
        <span class="pycomment"># just some shortcuts :P</span>
        surf = pygame.surfarray.pixels3d(screen)
        r    = self.rect
        <span class="pycomment"># hide selection rectangle</span>
        self.hide(screen)
        
        <span class="pycomment"># update background information</span>
        self.screen_backup[0] = copy.copy(surf[r[0]:r[0]+r[2],r[1]])
        self.screen_backup[1] = copy.copy(surf[r[0]:r[0]+r[2],r[1]+r[3]-1])
        self.screen_backup[2] = copy.copy(surf[r[0],r[1]:r[1]+r[3]])
        self.screen_backup[3] = copy.copy(surf[r[0]+r[2]-1,r[1]:r[1]+r[3]])

        <span class="pycomment"># draw selection rectangle:</span>
        surf[r[0]:r[0]+r[2],r[1]]        = self.col
        surf[r[0]:r[0]+r[2],r[1]+r[3]-1] = self.col
        surf[r[0],r[1]:r[1]+r[3]]        = self.col
        surf[r[0]+r[2]-1,r[1]:r[1]+r[3]] = self.col

        self.oldrect = r
        
        pygame.display.update(r)

    <span class="pydef">def hide(self,screen):</span>
        <span class="pycomment">""" hide(self,screen)
        This hides the selection rectangle using the stored background
        information. You usually call this after you're finished with the
        selection to hide the last rectangle.
        """</span>
        surf = pygame.surfarray.pixels3d(screen)
        x,y,x2,y2 = self.oldrect[0],self.oldrect[1],\
                    self.oldrect[0]+self.oldrect[2],\
                    self.oldrect[1]+self.oldrect[3]
        surf[x:x2,y   ] = self.screen_backup[0]
        surf[x:x2,y2-1] = self.screen_backup[1]
        surf[x,   y:y2] = self.screen_backup[2]
        surf[x2-1,y:y2] = self.screen_backup[3]
        pygame.display.update(self.oldrect)

<span class="pydef">def main():</span>
    <span class="pycomment">""" main()
    Simple test program for showing off the selection rectangles
    """</span>
    pygame.init()
    screen = pygame.display.set_mode((640,480),0,24)

    <span class="pycomment"># create a dotted background</span>
    surf = pygame.surfarray.pixels3d(screen)
    surf[:] = (255,255,255)
    surf[::4,::4] = (0,0,255)
    pygame.display.update()

    <span class="pycomment"># make up a test loop</span>
    finished = 0
    selection_on = 0
    while not finished:
        for e in pygame.event.get():
            if e.type == QUIT or (e.type == KEYDOWN and e.key == K_ESCAPE):
                finished = 1
            elif e.type == MOUSEBUTTONDOWN and e.button == 1:
                if not selection_on:
                    <span class="pycomment"># begin with selection as the user pressed down the left</span>
                    <span class="pycomment"># mouse button</span>
                    selection_on = 1
                    selection = SelectionRect(screen,e.pos)
            elif e.type == MOUSEMOTION:
                if selection_on:
                    <span class="pycomment"># update the selection rectangle while the mouse is moving</span>
                    selection.updateRect(e.pos)
                    selection.draw(screen)
            elif e.type == MOUSEBUTTONUP and e.button == 1:
                if selection_on:
                    <span class="pycomment"># stop selection when the user released the button</span>
                    selection_on = 0
                    rect = selection.updateRect(e.pos)
                    <span class="pycomment"># don't forget this!</span>
                    <span class="pycomment"># (or comment it out if you really want the final selection</span>
                    <span class="pycomment">#  rectangle to remain visible)</span>
                    selection.hide(screen)
                    <span class="pycomment"># just FYI</span>
                    print "<span class="pystring">Final selection rectangle:</span>",rect

    pygame.quit()

if __name__ == '<span class="pystring">__main__</span>': main()


</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

