<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Text Crystal Effect</H3>
            <p>Author: Andrew Meier<br>
            Submission date: December 14, 2001</p>
	  
            <p>Description: A function that renders the supplied text string with a bevelled or &quot;crystal&quot; effect.</p>
            
            <p align="center"><img src="crystal.png" width="548" height="78"></p>

            <p>Download: <a href="crystal.py">crystal.py</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: This code works by progressively sliding and darkening each edge in each of 8 directions, producing a bevelled, 3-D appearance.  It <i>is</i> quite slow, so it should be done during program start-up.  The included example script allows you to hit the 's' key; doing so will save the image to the current directory as a .bmp file.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">#!/usr/bin/env python</span>

<span class="pycomment">"""
Non-animated entry for the pygame text contest by Andrew A. Meier.

This effect was created using the 'Drop Shadow' example by Pete Shinners.

This code is very slow, and in desperate need of optimization but I think
the effect is pretty neat.
"""</span>

import os, sys, pygame, pygame.font, pygame.image
from pygame.locals import *


<span class="pydef">def shadeColor(color, amount):</span>
    <span class="pycomment">"""Brightens or darkens color by amount (-255 - 255)."""</span>
    r = color[0] + amount
    g = color[1] + amount
    b = color[2] + amount
    if r > 255: r = 255
    if r < 0:   r = 0
    if g > 255: g = 255
    if g < 0:   g = 0
    if b > 255: b = 255
    if b < 0:   b = 0
    return (r, g, b)


<span class="pydef">def textCrystal(font, message, bevel=5, fontcolor=(64, 128, 255), contrast=70):</span>
    <span class="pycomment">"""Renders text with a 'crystal' style apperance."""</span>
    base = font.render(message, 0, fontcolor)
    size = base.get_width() + bevel*2, base.get_height() + bevel*2+2
    img = pygame.Surface(size, 16)

    tl = (-1, -1)
    tc = (0, -1)
    tr = (1, -1)
    cr = (1, 0)
    br = (1, 1)
    bc = (0, 1)
    bl = (-1, 1)
    cl = (-1, 0)

    for x in range(-bevel, 1, 1):
        for position in (tl, tr, br, bl, tc, cr, bc, cl):
            for location in (tl, tr, br, bl, tc, cr, bc, cl):
                shade = ((-location[0])-location[1])*contrast
                img.blit(font.render(message, 1, shadeColor(fontcolor, shade)),
                         (bevel+location[0]+(x*position[0]), bevel+location[1]+(x*position[1])))
        img.blit(font.render(message, 1, fontcolor), (bevel, bevel))
    return img


entry_info = '<span class="pystring">Crystal, by Andrew Meier</span>'


<span class="pycomment">#this code will display our work, if the script is run...</span>
if __name__ == '<span class="pystring">__main__</span>':
    pygame.init()

    <span class="pycomment">#create our fancy text</span>
    white = 255, 255, 255
    grey = 100, 100, 100
    bigfont = pygame.font.Font(None, 60)
    text = textCrystal(bigfont, entry_info, 10, (64, 128, 255), 70)

    <span class="pycomment">#create a window the correct size</span>
    win = pygame.display.set_mode(text.get_size())
    winrect = win.get_rect()
    win.blit(text, (0, 0))
    pygame.display.flip()

    <span class="pycomment">#wait for the finish</span>
    while 1:
        event = pygame.event.wait()
        if event.type is KEYDOWN and event.key == K_s: <span class="pycomment">#save it</span>
            name = os.path.splitext(sys.argv[0])[0] + '<span class="pystring">.bmp</span>'
            print '<span class="pystring">Saving image to:</span>', name
            pygame.image.save(win, name)
        elif event.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
            break


</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

