<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Rainbow</H3>
            <p>Author: Mike Mulchek (god at seekrut.com)<br>
            Submission date: December 04, 2001</p>
	  
            <p>Description: A function to identify individual glyphs in a rendered text string, and make the glyph's pixels available for addressing.</p>
            
            <p align="center"><img src="rainbow.png" width="520" height="216"></p>

            <p>Download: <a href="rainbow.py">rainbow.py</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: The main GetGlyphs function is pretty hairy, and it's fairly slow, but it does a neat thing - it identifies individual glyphs in an arbitrary text surface, without reference to the original text string.  The example script doesn't really do justice to the power of this function; it just cycles each glyph through random colors.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">#!/usr/bin/env python</span>

<span class="pycomment">""" GetGlyphs is a function that will run through a given surface and retrieve all 
     shapes of a given color. It packs each shape into a list of coordinate pairs,
     and then packs them all up in a list. It does NOT recognize what shape it has
     found, but with some cleverness you could maybe make it into a cheap-OCR kind 
     of thing. 

     I wrote this so you can pick out the letters in a rendered surface and manipulate
     them independantly. 
     
     For shits and giggles, I make it make all the colors rainbow.  That could probably
     be made into a function, or GetGlyphs() could be extended to do a lot of things, 
     or ideally take a string and do stuff automagically. Whatever.

     Note: This is HORRENDOUSLY UNBELIEVABLY PHENOMENALLY SLOW. I have no idea what 
     I am doing and you are more than welcome to make it faster or rewrite it in one 
     line or delete it on sight for being ugly or whatever. On the bright side, 
     it does not crash anymore.
     
     Intention -- Mike Mulchek -- god@seekrut.com -- 12/04/2001 """</span>
     
     
import pygame, pygame.font, random
from pygame.locals import *

<span class="pydef">def GetGlyphs(surf, COLOR):</span>
     <span class="pycomment">""" Pull All Glyphs and Shapes of a certain color from a certain surface """</span>
     print "<span class="pystring">Looking for glyphs and shapes... This could take a few minutes...</span>"
     TempGlyph = []      <span class="pycomment"># Temporary Letter</span>
     GlyphList = []      <span class="pycomment"># Finished Glyph binder</span>
     Active = []         <span class="pycomment"># Scratch space</span>
     
     SurfCopy = pygame.Surface(surf.get_size())   <span class="pycomment"># Make a copy so we do not destroy anything</span>
     SurfCopy.blit(surf, (0,0))

     DONECOLOR = (random.randrange(10,250),random.randrange(10,250),random.randrange(10,250))
     if DONECOLOR == COLOR:
          DONECOLOR[0] += 1        <span class="pycomment"># Pick a scratch color that is guaranteed not to be the one we're looking for</span>

     for x in range(SurfCopy.get_width()):             <span class="pycomment"># Run over the entire surface until we find a pixel to start with</span>
          for y in range(SurfCopy.get_height()):
               if SurfCopy.get_at([x,y])[:3] == COLOR:      <span class="pycomment"># [:3] is to chop off the alpha info.. if true, start pulling the glyph!</span>
                    TempGlyph = []
                    Active = []
                    Active.append([x,y])
                    while len(Active):                      <span class="pycomment"># Check adjacent to a good-pixel in the order [Right Left Down Up]</span>
                         CurrPoint = Active.pop(0)               <span class="pycomment"># Keep adding good-color pixels...</span>
                         Point = [CurrPoint[0],CurrPoint[1]]     <span class="pycomment"># Throwing them into TempGlyph until there are no more to pull</span>
                         
                         try:
                              if SurfCopy.get_at(  [Point[0]+1, Point[1]]  )[:3] == COLOR:
                                   Active.append(  [Point[0]+1, Point[1]]    )
                                   SurfCopy.set_at([Point[0]+1, Point[1]], DONECOLOR)
                         except IndexError:
                              pass                         <span class="pycomment"># Sometimes you test a nonexistant surface index..</span> 

                         try:                            
                              if SurfCopy.get_at(  [Point[0]-1, Point[1]]  )[:3] == COLOR:
                                   Active.append(  [Point[0]-1, Point[1]]    )
                                   SurfCopy.set_at([Point[0]-1, Point[1]], DONECOLOR)
                         except IndexError:
                              pass                          <span class="pycomment"># but we don't care, so just ignore it....</span>

                         try:                              
                              if SurfCopy.get_at(  [Point[0], Point[1]+1]  )[:3] == COLOR:
                                   Active.append(  [Point[0], Point[1]+1]    )
                                   SurfCopy.set_at([Point[0], Point[1]+1], DONECOLOR)
                         except IndexError:
                              pass
                              
                         try:
                              if SurfCopy.get_at(  [Point[0], Point[1]-1]  )[:3] == COLOR:
                                   Active.append(  [Point[0], Point[1]-1]    )
                                   SurfCopy.set_at([Point[0], Point[1]-1], DONECOLOR)
                         except IndexError:
                              pass

                         TempGlyph.append(Point)                 <span class="pycomment"># When done searching the pixel, store it away</span>
                    GlyphList.append(TempGlyph)                  <span class="pycomment"># When the letter is done, save it in a binder</span>
                    pygame.display.flip()
     print "<span class="pystring">There are</span>", 
     print len(GlyphList),
     print "<span class="pystring">glyphs found altogether...</span>"
     return GlyphList                                            <span class="pycomment"># And when the entire surface is scrubbed, give the binder back</span>
     




if __name__ == '<span class="pystring">__main__</span>':    <span class="pycomment"># Whatever this does...</span>

     pygame.init()            <span class="pycomment"># Start the pygame</span>

     FALSE = 0                
     TRUE = 1

     ANTIALIAS = FALSE        <span class="pycomment"># Antialiasing sucks. </span>

     WHITE = (255,255,255)    <span class="pycomment"># Lots of pretty colors</span>
     BLACK = (0,0,0)
     RED = (255,0,0)
     GRAY = (128,128,128)
      
     OLDCOLOR = BLACK         <span class="pycomment"># OLDCOLOR is the color you want to search for</span>
     NEWCOLOR = RED           <span class="pycomment"># NEWCOLOR is the color you would like to change the glyph to</span>
     BACKGROUND = WHITE       <span class="pycomment"># BACKGROUND is the background color</span>
     
     FONTSIZE = 50            <span class="pycomment"># How big to make the font?</span>
     
     Messages = []            <span class="pycomment"># A list to keep your messages in</span>
     Surfaces = []            <span class="pycomment"># A list of surfaces the messages will render to</span>
          
     Messages.append("<span class="pystring">Hello there.</span>")     <span class="pycomment"># What do you want to say?  Add as many as you like.</span>
     Messages.append("<span class="pystring">How are you?</span>")
     Messages.append("<span class="pystring">This program is dull...</span>")
     Messages.append("<span class="pystring">But is pretty when you click?!</span>")

     CurrentFont = pygame.font.Font(None, FONTSIZE)  <span class="pycomment"># Set up a default font</span>

     for x in Messages:       <span class="pycomment"># Generate all of our messages</span>
          Surfaces.append(CurrentFont.render(x, ANTIALIAS, OLDCOLOR, BACKGROUND)) 


     <span class="pycomment"># A bunch of ugly crap to paste all the surfaces to the screen all lined up pretty</span>

     MaxWidth = []
     TotalHeight = 0
     CurrentHeight = 0

     for x in Surfaces:
          MaxWidth.append(x.get_width())

     for x in Surfaces:
          TotalHeight += x.get_height()

     screen = pygame.display.set_mode([max(MaxWidth),TotalHeight] )
     screen.fill(BACKGROUND)     

     for x in Surfaces:
          screen.blit(x, (0,CurrentHeight))
          CurrentHeight += x.get_height()
          
     pygame.display.flip()


     <span class="pycomment"># Finally, we are done with the test surface. Now to pull out the letters.</span>
     
     Glyphs = GetGlyphs(screen, OLDCOLOR)

     <span class="pycomment"># Super seekrut magic colorize</span>

     for G in Glyphs:
          RANDCOLOR = (random.randrange(0,256),random.randrange(0,256),random.randrange(0,256))
          for Point in G:
               screen.set_at(Point, RANDCOLOR)


     while pygame.event.poll().type != QUIT:
          if pygame.mouse.get_pressed()[0]:
               for G in Glyphs:
                    RANDCOLOR = (random.randrange(0,256),random.randrange(0,256),random.randrange(0,256))
                    for Point in G:
                         screen.set_at(Point, RANDCOLOR)
          pygame.display.flip()


</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

