<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title: Interference Pattern Effect </H3>
            <p>Author: Gareth Noyce (http://www.korruptor.demon.co.uk)<br>
            Submission date: July 11, 2002</p>
	  
            <p>Description: An effect that animates two copies of a concentric circle image to produce a nice interference effect.</p>

            <p align="center"><img src="interference.png" width="320" height="200"></p>

            <p>Download: <a href="interference.zip">interference.zip</a></p>	  

            <p>pygame version required: 1.3.4 with Surfarray<br>
            SDL version required: Any<br>
            Python version required: 2.0 with Numeric</p>

            <p>Comments: From the &quot;frigging-clever&quot; department comes interference.py, Gareth Noyce's latest Numeric-powered submission. Once two-color gif image is used, and Numeric is used to slice the image into screen-sized sections. These sections are then superimposed using Numeric's very fast add() function, and the result is a three-color image that can be redrawn very rapdily to produce a nice effect.  This script is a good example of Numeric used very well - slow for-loops are replaced by very fast slicing operations.</p>
        
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
Program		: interference.py
Author		: Gareth "Korruptor" Noyce -- http://www.korruptor.demon.co.uk
Desc		: 

                Yakk, Yakk. Interference Circles, more for the Amiga retro collection... :)

We load an image of concentric circles, whose dimensions are larger than the screen. Using
2 copies of the surfarray of this image; we take screen sized array-slices from each, moving
the offsets around a sin.  As the loaded image is a 2 colour gif, and the effect runs on an 8bit 
display, we can use Numeric's add() to sum the arrays, giving a simple surfarray to a custom
3 colour palette. Nice...

Play with the colours, change the image loaded (I was lazy by making a gif, but it does allow you
to play around with different numbers of circles or random shapes...) and adjust the sines to suit 
your taste....
"""</span>


import pygame, pygame.image
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *

<span class="pycomment"># ------------------------------------------------------------------------------------</span>

RES 		= (320,200)
PI 		= 3.14159
DEG2RAD 	= PI/180


<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def main():</span>
     
    	<span class="pycomment"># Initialise pygame, and grab an 8bit display.</span>
    	pygame.init()
    	screen		= pygame.display.set_mode(RES,0, 8)

        <span class="pycomment"># Setup the palette -- choice of colours is pretty important to getting the effect</span>
        <span class="pycomment"># to work well. Mine aren't that great, but then, this is a quickie ;-)</span>
        colour_map = zeros((256, 3))
        
        <span class="pycomment"># Background colour...</span>
        colour_map[0][0] =255
        colour_map[0][1] =255
        colour_map[0][2] =0

        <span class="pycomment"># Circles, not overlapping...</span>
        colour_map[1][0] =32
        colour_map[1][1] =255
        colour_map[1][2] =89

        <span class="pycomment"># Circles, overlapping...</span>
        colour_map[2][0] =225
        colour_map[2][1] =225
        colour_map[2][2] =148

        screen.set_palette(colour_map)
            
        <span class="pycomment"># Something to write to...</span>
        screen_buffer	= pygame.surfarray.array2d(screen)


        <span class="pycomment"># Load the Base image and make a couple of copies...</span>
    	ring1_surf	= pygame.image.load("<span class="pystring">ring1.gif</span>")
        ring1_arr 	= pygame.surfarray.array2d(ring1_surf)  
        ring2_arr	= ring1_arr


        xang = 0
        yang = 0
        xang1 = 0
        yang1 = 0

        x = 0
        y = 0
        x1 = 0
        y1 = 0

        <span class="pycomment"># Fruity loops...</span>
        while 1:
    
            <span class="pycomment"># Have we received an event to quit the program?</span>
            for e in pygame.event.get():
                if e.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
                    return


            <span class="pycomment"># Calculate the offsets for the slices of the two images...</span>
            offx = ((RES[0]/18)-10)*sin((xang*DEG2RAD) * 2.7)
            offy = ((RES[1]/13)-10)*cos((yang*DEG2RAD) * 2.5)
            offx1 = ((RES[0]/19)-12)*sin((xang1*DEG2RAD) * 2.2)
            offy1 = ((RES[1]/18)-12)*cos((yang1*DEG2RAD) * 2.5)

            <span class="pycomment"># Increment the positions of the slices...</span>
            x += offx
            y += offy
            x1 += offx1
            y1 += offy1

            xang += 1.2
            yang += 2.1
            xang1 += 1.5
            yang1 += 1.7

            <span class="pycomment"># Cut each slice from it's surfarray: [bp+offset:bp+offset+width, bp+offset:bp+offset+height]</span>
            foo = ring1_arr[40+int(x):40+int(x)+RES[0], 200+int(y):200+int(y)+RES[1]]
            bar = ring2_arr[80+int(x1):80+int(x1)+RES[0], 240+int(y1):240+int(y1)+RES[1]]
            
            <span class="pycomment"># Add the two arrays...</span>
            screen_buffer = add(foo,bar)

            <span class="pycomment"># Show results...</span>
            blit_array(screen, screen_buffer)
            pygame.display.flip()

        <span class="pycomment"># Bye!</span>
        print screen_buffer
        return

<span class="pycomment"># ------------------------------------------------------------------------------------
if __name__ == '<span class="pystring">__main__</span>': main()
<span class="pycomment"># End of sauce. Pass the chips...</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

