<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Mandelbrot Animation Example</H3>
            <p>Author: Pete Shinners (pete at shinners.org)<br>
            Submission date: April 20, 2001</p>
	  
            <p>Description: Animates into a python generated mandelbrot image</p>
            <p align="center"><img src="mandelbrot_screen.gif" width="320" height="240"></p>

            <p>Download: <a href="mandelbrot.zip">mandelbrot.zip</a></p>	  

            <p>pygame version required: Any with Numeric<br>
            SDL version required: Any<br>
            Python version required: &gt;= 2.0</p>

            <p>Comments: This code animates an exploration along the boundary of the Mandelbrot set. It requires Numeric, and accepts resolution and iterative depth parameters. Note that iterations greater than 9 will eventually produce an overflow error with the fast function; switch to the slow one for high depth values.</p>

            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">#!/usr/bin/env python</span>

<span class="pycomment">"""
Mandelbrot Animation
Pete Shinners
April 20, 2001

This is my attempt at doing a realtime python mandelbrot demonstation.
This mandelbrot algorithm orinally came from one of the Numeric python demos,
but I've gone and speeded it about about double. Still it's not what I'd
classify under "quick". The default settings of resolution 160x100 and
iterations of 6 runs reasonably quick on a Pentium2-300. increasing any of
these values looks really good, but slows things down a hunka lot!

Anyways, I'm don't entirely follow the mandelbrot algorithm myself. I'm
still not sure why this requires Complex number types, but my efforts to
replace with simple floats never worked. Owell, if anyone can speed this
up some, please share!

You may notice that while creating the mandelbrot it is created on one
axis, then transposed to be on the other. This was just a quicky addon
I did to make the mandelbrot appear in it's 'popular' orientation. You
could likely rework the algorithm a little so it just does it correctly,
but I don't imagine it would be any faster.

doh, i realized that in my "quicker" redo of this algorithm, the
complex numbers in the "Z" array will overflow with an iteration
value above 11. Too bad, cause it sure is pretty with that much detail.
Just to be fun then, I've included the unaltered mandelbrot function
which you can easily plugin and play with.
"""</span>


from Numeric import *
import pygame, pygame.surfarray
from pygame.locals import *
from Numeric import *
import sys



<span class="pydef">def old_slow_mandelbrot(LowX, HighX, LowY, HighY, stepx, stepy, maxiter):</span>
    xx=arange(LowX,HighX,(HighX-LowX)/stepx)
    yy=arange(HighY,LowY,(LowY-HighY)/stepy)*1j
    c=ravel(xx+yy[:,NewAxis])
    z=zeros(c.shape,Complex)
    output = zeros(c.shape) + 1
    for iter in range(maxiter):
        z=z*z+c
        finished=greater(abs(z),2.0)
        c=where(finished,0+0j,c)
        z=where(finished,0+0j,z)
        output=where(finished,iter,output)
    output.shape = (stepy, stepx)
    return transpose(output)



<span class="pydef">def new_fast_mandelbrot(LowX, HighX, LowY, HighY, stepx, stepy, maxiter):</span>
    <span class="pycomment">"creates a numeric array of the mandelbrot function"</span>
    if maxiter > 11: maxiter = 11  #for your own protection
    xx = arange(LowX, HighX, (HighX-LowX)/stepx)
    yy = arange(HighY, LowY, (LowY-HighY)/stepy) * 1.0j
    <span class="pycomment">#somtimes these arrays are too big by one element???</span>
    xx = xx[:stepx]
    yy = yy[:stepy]
    c = ravel(xx+yy[:,NewAxis])
    z = zeros(c.shape, Complex)
    output = zeros(c.shape) + 1
    for iter in range(maxiter):
        multiply(z, z, z)
        add(z, c, z)
        add(output, greater(abs(z), 2.0), output)
    output.shape = (stepy, stepx)
    return transpose(output)


<span class="pycomment">#you must choose wisely</span>

<span class="pycomment">#mandelbrot = old_slow_mandelbrot</span>
mandelbrot = new_fast_mandelbrot



<span class="pydef">def main():</span>
    if len(sys.argv) != 4:
        print '<span class="pystring">Arguments    : xsize, ysize, mandel_iterations</span>'
        print '<span class="pystring">Defaulting to: 160, 100, 6</span>'
        res = 160, 100
        iterations = 6
    else:
        res = int(sys.argv[1]), int(sys.argv[2])
        iterations = int(sys.argv[3])
        print '<span class="pystring">Arguments    :</span>', res, iterations

    pygame.init()
    screen = pygame.display.set_mode(res, 0, 8)

    l_val, r_val = -2.1, 0.7
    t_val, b_val = -1.2, 1.2

    l_inc, r_inc = .03, -.02
    t_inc, b_inc = .03, -.005
    while 1:
        if pygame.event.peek([QUIT,KEYDOWN]): break

        mand = mandelbrot(l_val, r_val, t_val, b_val, res[0], res[1], iterations)
        multiply(mand, 36, mand) #this just chooses 'prettier' color index values
        pygame.surfarray.blit_array(screen, mand)
        pygame.display.flip()

        l_val += l_inc
        r_val += r_inc
        t_val += t_inc
        b_val += b_inc



if __name__ == '<span class="pystring">__main__</span>': main()

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

