<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Meltdown</H3>
            <p>Author: Sebastian &quot;WhiteTiger&quot; John (TheWhiteTiger at gmx.net)<br>
            Submission date: September 29, 2002</p>
	  
            <p>Description: A port of the 'Melt' effect.</p>
            <p align="center"><img src="meltdown.jpg" width="320" height="240"></p>

            <p>Download: <a href="meltdown.zip">meltdown.zip</a></p>	  

            <p>pygame version required: <br>
            SDL version required: <br>
            Python version required: </p>

            <p>Comments: This script ports the well-known 'melt' effect, familiar to users of the 'xmelt' program, and to players of Doom.</p>
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
Name:		Meltdown
Author:		Sebastian "WhiteTiger" John
Email:		TheWhiteTiger@gmx.net

This is public domain code.

Meltdown -- 'mess up the screen' (cite by David Lemke). This effect takes a
surface, moves blocks from the top downwards and fills the place where the
block has been with black until the whole surface is black. If you want to
use it in your own code, have a look at the definition of 'main()'.

This is a port of the linux/Xlib program 'xmelt' by Dave Lemke. I did almost
nothing more than porting the C/Xlib stuff to python/pygame. Unfortunately
my port is much slower than the original code... That means that it doesn't
need any pygame.time stuff at all ;)
"""</span>

import random
import pygame
from pygame.locals import *

<span class="pydef">def myrandom(n):</span>
    <span class="pycomment"># another layer of overhead...</span>
    return random.randrange(n)

<span class="pydef">class Meltdown:</span>
    <span class="pydef">def __init__(self, surface, min_size=10, max_size=100, min_dist=10,
            min_width=30, width_add=20, finished=50):</span>
        
        self.surf = surface
        
        self.WIDTH, self.HEIGHT = surface.get_size()
        
        self.MIN_SIZE = min_size
        self.MAX_SIZE = max_size
        
        self.MIN_DIST = min_dist
        
        self.MIN_WIDTH = min_width
        self.WIDTH_ADD = width_add
        
        self.MAX_HEIGHT = self.HEIGHT - self.MIN_SIZE
        self.FINISHED = finished
        
        self.heights = []
        while len(self.heights) &lt; self.WIDTH: self.heights.append(0)
        
        self.finished = 0
    
    <span class="pycomment"># this procedure is only for historical reasons
    # it is now inlined because of speed
    #def calc_xloc(self, width):
    #    # original version
    #    #xloc = rand(self.WIDTH + self.MIN_WIDTH) - self.MIN_WIDTH
    #    #xloc = rand(self.WIDTH) - self.MIN_WIDTH
    #    #xloc = rand(self.WIDTH) - 2*self.MIN_WIDTH
    #    #xloc = rand(self.WIDTH)
    #    # this one seems to look best:
    #    xloc = rand(self.WIDTH + self.MIN_WIDTH) - 2*self.MIN_WIDTH
    #    return max(min((xloc + width), (self.WIDTH - width)), 0)</span>
    
    <span class="pydef">def step(self):</span>
        <span class="pycomment"># redefinitions to speed things up a little bit</span>
        WIDTH = self.WIDTH
        MIN_WIDTH = self.MIN_WIDTH
        HEIGHT = self.HEIGHT
        MAX_HEIGHT = self.MAX_HEIGHT
        
        rand = myrandom
        heights = self.heights
        surf = self.surf
        
        _range = range
        _min = min
        _max = max
        
        <span class="pycomment"># precalculations</span>
        width = rand(MIN_WIDTH) + self.WIDTH_ADD
        
        <span class="pycomment">#xloc = self.calc_xloc(width)</span>
        xloc = rand(WIDTH + MIN_WIDTH) - 2*MIN_WIDTH
        xloc = _max(_min((xloc + width), (WIDTH - width)), 0)
        
        yloc = HEIGHT
        for i in _range(xloc, (xloc + width)):
            yloc = _min(yloc, heights[i])
        if yloc == HEIGHT:
            return surf, []
        
        <span class="pycomment"># calculate block size</span>
        dist = rand(yloc/10 + self.MIN_DIST)
        size = rand(_max(yloc + self.MIN_SIZE, self.MAX_SIZE))
        
        <span class="pycomment"># define rects for blitting</span>
        destpos = (xloc, yloc + dist)
        destrect = (xloc, yloc, width, dist)
        sourcerect = (xloc, yloc, width, size)
        
        <span class="pycomment"># the visible bit</span>
        surf.blit(surf, destpos, sourcerect)
        surf.fill((0, 0, 0), destrect)
        
        <span class="pycomment"># postcalculations</span>
        yloc += dist
        for i in _range(xloc, (xloc + width)):
            if heights[i] &lt; MAX_HEIGHT and yloc >= MAX_HEIGHT:
                self.finished += 1
            heights[i] = _max(heights[i], yloc)
        
        if self.finished >= (WIDTH - self.FINISHED):
            return None, []
        
        <span class="pycomment"># everything is fine, return the new surface</span>
        return surf, [sourcerect, destrect]

<span class="pydef">def main():</span>
    <span class="pycomment"># resolution</span>
    <span class="pycomment">#RES = (640, 480)</span>
    RES = (320, 240)
    <span class="pycomment">#RES = (160, 120)</span>
    
    <span class="pycomment"># pygame initialization</span>
    pygame.init()
    screen = pygame.display.set_mode(RES)
    
    <span class="pycomment"># load and convert an image</span>
    image = pygame.image.load("<span class="pystring">back.jpg</span>").convert()
    image = pygame.transform.scale(image, RES)
    
    <span class="pycomment"># blit the surface to the screen</span>
    screen.blit(image, (0,0))
    pygame.display.flip()
    
    <span class="pycomment"># instantiate the Meltdown class</span>
    meltdown = Meltdown(image)
    
    <span class="pycomment"># first step</span>
    surface, dirty = meltdown.step()
    
    <span class="pycomment"># main loop</span>
    while surface:
        <span class="pycomment"># event stuff</span>
        for ev in pygame.event.get():
            if ev.type in (QUIT, KEYDOWN, MOUSEBUTTONDOWN):
                return
        
        <span class="pycomment"># redraw screen</span>
        screen.blit(surface, (0,0))
        pygame.display.update(dirty)
        
        <span class="pycomment"># next step</span>
        surface, dirty = meltdown.step()

if __name__ == "<span class="pystring">__main__</span>":
    main()


</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

