<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title: Motion Blur Effect </H3>
            <p>Author: Gareth Noyce (http://www.korruptor.demon.co.uk)<br>
            Submission date: August 4, 2002</p>
	  
            <p>Description: A nice motion blur effect using Numeric.</p>

            <p align="center"><img src="motion_blur.png" width="320" height="200"></p>

            <p>Download: <a href="pygame-motionblur.zip">pygame-motionblur.zip</a></p>	  

            <p>pygame version required: Any with Surfarray<br>
            SDL version required: Any<br>
            Python version required: 2.0 with Numeric</p>

            <p>Comments: Mr. Noyce has created an attractive motion blur effect in 8-bit colour, using Numeric arrays as his accumulator.  Make sure to read the included readme file; &quot;burn&quot; and truecolor effects are possible, though increasing the bit depth will reduce your framerate.</p>
        
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
Author		Gareth "Korruptor" Noyce
Email		korruptor@mac.com
Web		http://www.korruptor.demon.co.uk
Depnd		Numeric, Surfarray
Desc:

In this example, we create 12 blank surfarrays in the list:
'surface'. These will be used to store historic frames. In addition 
we create an 'acculumator' surfarray, which is again blank... 

Each time we render a new frame;

* we subtract the pixel values in  the 'oldest' surfarray from the 
  'accumulator', 
* blit a few sprites onto a temporary surface [tmp], 
* convert this temporary surface to a surfarray and store it in 
  place of the 'oldest' surfarray in our list. 
* we then add the pixel values of the newly created surfarray to 
  the accumulator and blit the accumulator to screen. 

Note the use of fixed, ordered palettes which handle the end display.
"""</span>


import pygame, pygame.image
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *

<span class="pycomment"># ------------------------------------------------------------------------------------</span>

RES 		= (320,200)
PI 		= 3.14159
DEG2RAD 	= PI/180
NUM_SURFS 	= 12

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def main():</span>
     
    	<span class="pycomment"># Initialise pygame, and grab an 8bit display.</span>
    	pygame.init()

    	screen 		= pygame.display.set_mode(RES,0, 8)
        tmp		= screen
        accumulator 	= tmp

        <span class="pycomment"># Load our sprites...</span>
        pal 		= pygame.image.load("<span class="pystring">palette.gif</span>")
        background 	= pygame.image.load("<span class="pystring">background.gif</span>")
        sprite		= pygame.image.load("<span class="pystring">sprite.gif</span>")
        sprite2		= pygame.image.load("<span class="pystring">sprite2.gif</span>")
    

        <span class="pycomment"># Create a series of blank "frames" which we use to record our blits.</span>
        surface 	= []
        oldest 		= 0
        for i in range(0,NUM_SURFS):
                surface.append(zeros(RES))
        
        <span class="pycomment"># Get everything singing from the same palette</span>
        tmp.set_palette(pal.get_palette())
        screen.set_palette(pal.get_palette())
        sprite.set_palette(pal.get_palette())
        sprite2.set_palette(pal.get_palette())
        sprite.set_colorkey([0,0,0])
        sprite2.set_colorkey([0,0,0])
        background.set_palette(pal.get_palette())

        <span class="pycomment"># The accumulator holds the "final" blit, so set this up as a blank array for now</span>
        accumulator = pygame.surfarray.array2d(accumulator)

        xa =0
        ya =0
        xa1=0
        ya1=0
        xa2=0
        ya2=0

        <span class="pycomment"># Fruity loops...</span>
        while 1:
    
            <span class="pycomment"># Have we received an event to quit the program?</span>
            for e in pygame.event.get():
                if e.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
                    return

            <span class="pycomment"># Subtract the oldest frame from the accumulator...</span>
            accumulator = subtract(accumulator,surface[oldest])

            <span class="pycomment"># Now, draw a new frame...</span>
            x    = ((RES[1]/10))*cos((xa*DEG2RAD) * 3)
            y    = ((RES[1]/4))*cos((ya*DEG2RAD))
            x1   = ((RES[1]/3))*cos((xa1*DEG2RAD)*1.7)
            y1   = ((RES[1]/3))*cos((ya1*DEG2RAD)*2.5)
            x2   = (RES[1]/2)*cos((xa2*DEG2RAD))
            y2   = (RES[1]/2)*cos((ya2*DEG2RAD))
            xa  += 1.5
            ya  += 3
            xa1 += 1.2
            ya1 += 2.75
            xa2 += 2.3
            ya2 += 4.8

            tmp.blit(background,(0,0))
            tmp.blit(sprite,(x+20,y+50))
            tmp.blit(sprite2,(x1+140,y1+75))
            tmp.blit(sprite2,(x2+140,y2+75))

            <span class="pycomment"># And replace the oldest frame with our new one...</span>
            surface[oldest] = pygame.surfarray.array2d(tmp)

            <span class="pycomment"># Add this new frame to the accumulator...</span>
            accumulator = add(accumulator,surface[oldest])

            <span class="pycomment"># Inc the pointer to the next oldest frame...</span>
            oldest = (oldest + 1)%NUM_SURFS

            <span class="pycomment"># ...and blit the accumulator to the screen before repeating...</span>
            blit_array(screen, accumulator)
            pygame.display.flip()

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
if __name__ == '<span class="pystring">__main__</span>': main()
<span class="pycomment"># End of sauce. Pass the chips...</span>


</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

