<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Fast Circle Drawing Using Surfarray</H3>
            <p>Author: Pete Shinners (pete at shinners.org)<br>
            Submission date: May 19, 2001</p>
	  
            <p>Description: Pete says, &quot;I put together a quick demonstration on how to make your own circles using Numeric python and the surfarray modules.  Note that this code only does circles. Changing it to handle ellipses shouldn't be too difficult, but you'll need to be a little up on your math.  Remember you'll need Numeric python installed ( <a href="http://www.pfdubois.com/numpy/">www.pfdubois.com</a> ), but other than that you should be able to pick and learn at my little circle drawing example.&quot;</p>

            <p align="center"><img src="circle.gif" width="200" height="200"></p>

            <p>Download: <a href="circle.tar.gz">circle.tar.gz</a></p>	  
            <p>Download: <a href="circle.zip">circle.zip</a></p>
            
            <p>pygame version required: Any with SurfArray<br>
            SDL version required: Any<br>
            Python version required: Any with Numeric</p>

            <p>Comments: Pete gives us a circle drawing routine, and a small script to test it. The math is pretty straightforward, but the syntax may be a little unfamiliar to those not used to working with Numpy.</p>
            <p>As this script draws its circles, you may note that many look a little oblong. This is most noticable with the smallest , brightest circles, but all look a little squared off. This occurs not because of any particular weakness in the code, but rather because of the lack of anti-aliasing.  Also note that this code doesn't run as fast as it could - Pete has added an artificial delay loop at the end of his script. Comment out the last line (pygame.time.delay(100)) to see how fast it can go.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""Draw a circle with Numeric"""</span>

<span class="pycomment">#quick and dirty imports</span>
import pygame
from pygame.locals import *
import pygame.surfarray as surfarray
from Numeric import *
squareroot = sqrt


<span class="pydef">def makecircle(radius, color):</span>
    <span class="pycomment">"make a surface with a circle in it, color is RGB"</span>

    <span class="pycomment">#make a simple 8bit surface and colormap</span>
    surf = pygame.Surface((radius*2, radius*2), 0, 8)
    surf.set_palette(((0, 0, 0), color))

    <span class="pycomment">#first build circle mask</span>
    axis = abs(arange(radius*2)-(radius-0.5)).astype(Int)**2
    mask = squareroot(axis[NewAxis,:] + axis[:,NewAxis])
    mask = less(mask, radius)

    surfarray.blit_array(surf, mask)    <span class="pycomment">#apply circle data</span>
    surf.set_colorkey(0, RLEACCEL)      <span class="pycomment">#make transparent</span>

    return surf


if __name__ == '<span class="pystring">__main__':</span>
    <span class="pycomment">#lets do a little testing</span>
    from random import *
    pygame.init()
    screen = pygame.display.set_mode((200, 200))
    while not pygame.event.peek([QUIT,KEYDOWN]):
        radius = randint(10, 20)
        pos = randint(0, 160), randint(0, 160)
        color = randint(20, 200), randint(20, 200), randint(20, 200)
        circle = makecircle(radius, color).convert()
        screen.blit(circle, pos)
        pygame.display.update((pos, (radius*2, radius*2)))
        pygame.time.delay(100)

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

