<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Fractal Example</H3>
            <p>Author: Pete Shinners (pete at shinners.org)<br>
            Submission date: April 19, 2001</p>
	  
            <p>Description: An example of animated colormaps</p>
            <p align="center"><img src="fractal_screenshot.jpg" width="320" height="240"></p>

            <p>Download: <a href="fractal.zip">fractal.zip</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: 2.0</p>

            <p>Comments: Palette cycling or shifting is one of the oldest computer animation techniques. When hardware speeds didn't permit large blits in real time, a developer would often simulate movement by changing a display's underlying palette. Pete's example demonstrates the principle - first you blit a multicoloured (even grayscale, in this case) image once, then cycle the display's palette. As Pete notes, no subsequent blits are necessary; you don't even need to call update() or flip(). Simply calling set_palette() in a tight loop creates the illusion of motion. This technique is suitable for many static sprite-like animation effects - flowing waterfalls, rolling waves, etc.</p>

            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">#!/usr/bin/env python</span>

<span class="pycomment">"""
Palette animation example

Pete Shinners
April 19, 2001

In this example we are loading a greyscale fractal like
image. We then create an animated colormap and apply it to
the display. Note we could probably use Numeric or something
to create the color palette a bit quicker and easier, but in
this case it's fast enough to do it with python.

One thing I noticed is that on windows, the "windib" video
device runs over twice as fast as the "directx" one. Go figure.
"""</span>

import pygame, pygame.image
from pygame.locals import *



<span class="pydef">def build_palette(step):</span>
    <span class="pycomment">"build a palette. that is a list with 256 RGB triplets"</span>
    loop = range(256)
    <span class="pycomment">#first we create a 256-element array. it goes from 0, to 255, and back to 0</span>
    ramp = [abs((x+step*3)%511-255) for x in loop]
    <span class="pycomment">#using the previous ramp and some other crude math, we make some different</span>
    <span class="pycomment">#values for each R, G, and B color planes</span>
    return [(ramp[x], ramp[(x+32)%256], (x+step)%256) for x in loop]


<span class="pydef">def main():</span>
    <span class="pycomment">"run the example"</span>

    <span class="pycomment">#initialize, note we want an 8bit display here</span>
    pygame.init()
    screen = pygame.display.set_mode((640, 480), HWSURFACE|HWPALETTE, 8)

    <span class="pycomment">#create a basic palette to start with</span>
    screen.set_palette(build_palette(0))

    <span class="pycomment">#print a little information</span>
    print '<span class="pystring">SCREEN: hardware=%d, depth=%d</span>' % \
          (screen.get_flags()&HWSURFACE, screen.get_bitsize())

    <span class="pycomment">#load our image and put it on the screen</span>
    img = pygame.image.load('<span class="pystring">fractal.gif</span>')
    screen.blit(img, (0, 0))
    pygame.display.flip()


    <span class="pycomment">#loop, simply create a new colormap with a different input value</span>
    step = 0
    showstoppers = QUIT, KEYDOWN, MOUSEBUTTONDOWN
    while pygame.event.poll().type not in showstoppers:
        palette = build_palette(step)
        screen.set_palette(palette)
        <span class="pycomment">#note that we needn't flip() or update() the</span>
        <span class="pycomment">#display when only changing the palette</span>
        step += 1



if __name__ == '<span class="pystring">__main__</span>':main()    

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

