<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title: Starfield </H3>
            <p>Author: Jon Parise (jon at csh.rit.edu)<br>
            Submission date: May 13, 2001</p>
	  
            <p>Description: Displays a simple moving parallax starfield.</p>

            <p>Download: <a href="starfield.zip">starfield.zip</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: Mr. Parise's code displays a starfield, and creates the illusion of movement through parallax effects. The direction of movement can be changed by pressing the left and right arrow keys, and two different movement speeds are available (hit the down or up arrow keys). Hit Esc to quit.</p>

            <p>This example demonstrates that it is possible to draw and erase individual pixels quickly using pygame. If you need to update the whole screen, the key is to do as Mr. Parise does below - call pygame.display.update() only once per frame; calling it after every drawing operation is wasteful and produces flicker.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">#!/usr/bin/env python

"""starfield, by Jon Parise (jon at csh.rit.edu)

Simple program to display a moving parallax starfield.

The speed of the starfield can be altered using the up and down arrows.
The direction of the starfield can be changed using the left and right arrows.

The algorythm is based on code by Sean Bugel (bugelsea at rcn.com).
"""</span>

import random
import pygame
from pygame.locals import *

<span class="pycomment"># Constants</span>
NUM_STARS = 40
SCREEN_SIZE = [640, 480]
WHITE = 255, 255, 255
BLACK = 20, 20, 40
LIGHTGRAY = 180, 180, 180
DARKGRAY = 120, 120, 120
LEFT = 0
RIGHT = 1

<span class="pydef">def initStars(screen):</span>
    <span class="pycomment">"Create the starfield"</span>

    <span class="pycomment"># The starfield is represented as a dictionary of x and y values.</span>
    stars = []

    <span class="pycomment"># Create a list of (x,y) coordinates.</span>
    for loop in range(0, NUM_STARS):
        star = [random.randrange(0, screen.get_width() - 1),
                random.randrange(0, screen.get_height() - 1)]
        stars.append(star);

    return stars

<span class="pydef">def moveStars(screen, stars, start, end, direction):</span>
    <span class="pycomment">"Correct for stars hitting the screen's borders"</span>

    for loop in range(start, end):
        if (direction == LEFT):
            if (stars[loop][0] <> 1):
                stars[loop][0] = stars[loop][0] - 1
            else:
                stars[loop][1] = random.randrange(0, screen.get_height() - 1)
                stars[loop][0] = screen.get_width() - 1
        elif (direction == RIGHT):
            if (stars[loop][0] <> screen.get_width() - 1):
                stars[loop][0] = stars[loop][0] + 1
            else:
                stars[loop][1] = random.randrange(0, screen.get_height() - 1)
                stars[loop][0] = 1

    return stars

<span class="pydef">def main():</span>
    <span class="pycomment">"Main starfield code"</span>

    random.seed()

    <span class="pycomment"># Initialize the pygame library.</span>
    pygame.init()
    screen = pygame.display.set_mode(SCREEN_SIZE, HWSURFACE|DOUBLEBUF|FULLSCREEN)
    <span class="pycomment">#screen = pygame.display.set_mode(SCREEN_SIZE, HWSURFACE|DOUBLEBUF)</span>
    pygame.display.set_caption("<span class="pystring">Starfield</span>")
    pygame.mouse.set_visible(0)

    <span class="pycomment"># Set the background to black.</span>
    screen.fill(BLACK)

    <span class="pycomment"># Simulation variables.</span>
    delay = 8
    inc = 2
    direction = LEFT

    <span class="pycomment"># Create the starfield.</span>
    stars = initStars(screen)

    <span class="pycomment"># Place ten white stars</span>
    for loop in range(0, 10):
        screen.set_at(stars[loop], WHITE)

    <span class="pycomment"># Main loop</span>
    while 1:

        <span class="pycomment"># Handle input events.</span>
        event = pygame.event.poll()
        if (event.type == QUIT):
            break
        elif (event.type == KEYDOWN):
            if (event.key == K_ESCAPE):
                break
            elif (event.key == K_UP):
                if (delay >= 2): delay = delay - 1
            elif (event.key == K_DOWN):
                if (delay <= 16): delay = delay + 1
            elif (event.key == K_LEFT):
                direction = LEFT
            elif (event.key == K_RIGHT):
                direction = RIGHT 

        <span class="pycomment"># Used to slow down the second and third field.</span>
        inc = inc + 1

        <span class="pycomment"># Erase the first star field.</span>
        for loop in range(0, 10):
            screen.set_at(stars[loop], BLACK)

        <span class="pycomment"># Check if first field's stars hit the screen border.</span>
        stars = moveStars(screen, stars, 0, 10, direction)

        <span class="pycomment"># Second star field algorythms.</span>
        if (inc % 2 == 0):

            <span class="pycomment"># Erase the second field.</span>
            for loop in range(11, 20):
                screen.set_at(stars[loop], BLACK)

            <span class="pycomment"># Checks to see if the second field's stars hit the screen border.</span>
            stars = moveStars(screen, stars, 11, 20, direction)

            <span class="pycomment"># Place ten light gray stars.</span>
            for loop in range(11, 20):
                screen.set_at(stars[loop], LIGHTGRAY)

        <span class="pycomment"># Third star field algorythms.</span>
        if (inc % 5 == 0):
           
            <span class="pycomment"># Erase the third field.</span>
            for loop in range(21, NUM_STARS):
                screen.set_at(stars[loop], BLACK)

            <span class="pycomment"># Checks to see if the third field's stars hit the screen border.</span>
            stars = moveStars(screen, stars, 21, NUM_STARS, direction)

            <span class="pycomment"># Place ten dark gray stars.</span>
            for loop in range(21, NUM_STARS):
                screen.set_at(stars[loop], DARKGRAY)

        <span class="pycomment"># Place ten white stars.</span>
        for loop in range(0, 10):
            screen.set_at(stars[loop], WHITE)

        <span class="pycomment"># Control the starfield speed.</span>
        pygame.time.delay(delay)

        <span class="pycomment"># Make sure this variable doesn't get too large.</span>
        if (inc == 500): inc = 2

        <span class="pycomment"># Update the screen.</span>
        pygame.display.update()

<span class="pycomment"># Start the program.</span>
if __name__ == '<span class="pystring">__main__</span>': main()

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

