<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Roto-Zoom Effect</H3>
            <p>Author: Gareth Noyce (g at korruptor.demon.co.uk)<br>
            Submission date: March 8, 2002</p>
	  
            <p>Description: Rotates, zooms and tiles a bitmap.</p>
            
            <p align="center"><img src="roto_zoom.png" width="320" height="240"></p>

            <p>Download: <a href="roto_zoom-complete.zip">roto_zoom-complete.zip</a></p>	  

            <p>pygame version required: Any (with Surfarray)<br>
            SDL version required: Any<br>
            Python version required: Any (with Numeric)</p>

            <p>Comments: More effect craziness from Gareth Noyce.  This code performs an effect you've probably seen somewhere before - it tiles a bitmap across the window, then rotates it, and zooms in and out.  The code works pixel-by-pixel to generate each successive frame; it would be interesting to contrast the speed of this effect with one based on pygame's own transform module.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""roto_zoom.py -- Korruptor Jan 2001, test code for Seal Basher (v1.0)

This is a direct port of Allegro source written by "Amarillion" (amarillion@yahoo.com)!
I did not 'design', 'write' or figure this out! :-) But I sure did learn alot by playing
with it...

I also borrowed Pete Shinners code for scaling and blitting a surfarray (as featured in
his fast flame effect) as I've only calculated an 1/8th of the screen estate for speed.

--
The following script (and function) is the old 'roto_zoom' demo effect as seen pretty much
everywhere in the mid-late 90s. Why do this when pygame has scale and rotate functions?
Well, it's useful to know some of the tricks of cos and sin (IMHO) and this is me trying
to learn as much about basic maths and coordinate systems as I can... I figure someone
else might benefit from this as well... (Although for most of the ninja coders on #pygame
this is probably blindingly obvious, at least I'm catching up tho!)

The function is fairly simple, copying pixels from a source bitmap to a larger destination 
bitmap, where the destination pixel's source is calculated via cos (for X) and sin (for y) --
these calculated values are used as incrementors for the position to be copied from a source
surfarray, and with this information (and some modulo arithmetic to keep the position in bounds) 
we can create a tiling, rotated and scaled bitmap that will fit a much larger area.

The main function handles incrementing the angle and scale factor, and repeatedly 
calls the "roto_zoom" function with these new values to provide the animation...

And that as they say, is about that. 

As you can see from this example, choice of source bitmap is fairly important. Monty doesn't
really scale too well...
"""</span>

import pygame, pygame.transform, pygame.image
from operator import *
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pycomment"># Glob decs</span>

<span class="pycomment"># Screen resolution...</span>
RES 	= array((640,480))

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def main():</span>
    <span class="pycomment">"Inisalises display, precalculates the cosine values, and controls the update loop"</span>
    
    <span class="pycomment"># Initialise pygame, and grab an 8bit display.</span>
    pygame.init()
    screen_surface = pygame.display.set_mode(RES, 0, 8)

    <span class="pycomment"># Load a source image, and convert it to a surfarray...</span>
    image = pygame.image.load("<span class="pystring">monty.gif</span>")
    image_buff = pygame.surfarray.array2d(image)

    <span class="pycomment"># Numeric array (working) for the display. Do all our fun stuff here...</span>
    rot_buffer  = zeros(RES/8)
    
    <span class="pycomment"># Pygame Surface object which will take the surfarray data and be translated into a screen blit... </span>
    rot_surface = pygame.Surface((RES[0]/8, RES[1]/8), 0, 8)
        
    <span class="pycomment"># apply the same palette to plasma_surface</span>
    rot_surface.set_palette(image.get_palette())

    <span class="pycomment"># Setup our parameters for calling the roto_zoom function...</span>
    angle = 0.1
    scale = 2
    surface_x = RES[0]/8
    surface_y = RES[1]/8
    image_x = image.get_width()
    image_y = image.get_height()

    <span class="pycomment"># Angle incrementor</span>
    inc = 0.1

    <span class="pycomment"># Zoom up or down...</span>
    dir = 0

    <span class="pycomment"># Fruity loops...</span>
    while 1:
    
        <span class="pycomment"># Have we received an event to close the window?</span>
        for e in pygame.event.get():
            if e.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
                return

	<span class="pycomment"># Call the roto_zoom function, blit a scaled surfarray to the screen and display it..</span>            
	<span class="pycomment"># We mod the angle to keep it in bounds...</span>
	rot_buffer = roto_zoom(rot_buffer, image_buff, mod(angle,360), scale, surface_x, surface_y, image_x, image_y) 
	blit_scaled_surface(screen_surface, rot_buffer, rot_surface)
	pygame.display.update()

        <span class="pycomment"># Increment the angle for the next iteration</span>
    	angle += inc

        <span class="pycomment"># Check the scale is in bounds, and increment or decrement accordingly...</span>
	if(dir):
	    scale -= 0.1
	    if(scale <= 0):
		dir = 0
        else:
	    scale += 0.1
            if(scale >= 4):
            	dir = 1
            
<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def roto_zoom(dest, src, angle, scale, x_size, y_size, x_mask, y_mask):</span>
    <span class="pycomment">""" Function to rotate and scale a supplied surfarray, filling the screen with the results..."""</span>
 
    <span class="pycomment"># Our initial positions in the source surfarray</span>
    src_x = 0.0
    src_y = 0.0

    <span class="pycomment"># Our initial positions in the destination surfarray</span>
    dest_x = 0
    dest_y = 0

    <span class="pycomment"># The "delta" values for incrementing the position of src_x & src_y</span>
    dx = 0.0
    dy = 0.0

    <span class="pycomment"># Src_x & src_y will be initialised to start_x and start_y at the beginning of each newline</span>
    start_x = 0.0
    start_y = 0.0

    <span class="pycomment"># Calculate the increments we need for the source bitmap everytime we move right a pixel in the destination</span>
    dx = cos(angle) * scale
    dy = sin(angle) * scale

    <span class="pycomment"># For each row in the destination</span>
    for dest_y in range(0,y_size):
       
        <span class="pycomment"># Set the position in the source to the start of this line</span>
	src_x = start_x
	src_y = start_y
	
	<span class="pycomment"># For each pixel in the row...</span>
	for dest_x in range(0,x_size):
		
		<span class="pycomment"># copy a pixel from the source to the desitination</span>
		<span class="pycomment"># We mod the src_x/y vars to keep them in bounds of the source image...</span>
		dest[dest_x][dest_y] = src[mod(int(src_x),x_mask)][mod(int(src_y),y_mask)]

		<span class="pycomment"># Increment the source positions by our calculated increments...</span>
		src_x += dx
		src_y += dy

	<span class="pycomment"># Change the starting positions for the next line (this is a dot product I think...)</span>
    	start_x -= dy
   	start_y += dx

    <span class="pycomment"># Return the surfarray for blitting...</span>
    return dest


<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def blit_scaled_surface(screen, flame, miniflame):</span>
    <span class="pycomment">"double the size of the data, and blit to screen -- Nicked from Shread's Fast Flame"</span>
    blit_array(miniflame, flame)
    s2 = pygame.transform.scale(miniflame, screen.get_size())
    screen.blit(s2, (0,0))

<span class="pycomment"># Shit captain! In the void of space we're all alone...</span>
<span class="pycomment"># Don't worry ensign. I have a plan: Fire our trippy laser!</span>
if __name__ == '<span class="pystring">__main__</span>': main()

<span class="pycomment"># End of sauce. Pass the chips...</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

