<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Screen dimmer class</H3>
            <p>Author: Tobias Thelen (tthelen at uni-osnabrueck.de)<br>
            Submission date: September 6, 2001</p>
	  
            <p>Description: A class for 'dimming' (i.e. darkening) the entire screen</p>
            
            <p>Download: <a href="dimmer.py">dimmer.py</a></p>	  

            <p>pygame version required: Any<br>
            SDL version required: Any<br>
            Python version required: Any</p>

            <p>Comments: Comments: This class is useful for a variety of effects in any non-palettized screen mode.  Consider using a quick flash to white with explosion effects, or a doom-like flash to red when the player is injured.  Mr. Thelen has included an undim function, so you don't need to manually save and restore your screen after the fade.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
Dimmer class

Tobias Thelen (tthelen@uni-osnabrueck.de)
6 September 2001

PUBLIC DOMAIN
Use it in any way you want...

tested with: Pyton 2.0/pygame-1.1, Windows 98

A class for 'dimming' (i.e. darkening) the entire screen, useful for:
- indicating a 'paused' state
- drawing user's attention away from background to e.g. a Quit/Don't Quit
  dialog or a highscore list or...

Usage:

dim=Dimmer(keepalive=1)
  Creates a new Dimmer object,
  if keepalive is true, the object uses the same surface over and over again,
  blocking some memory, but that makes multiple undim() calls possible - 
  Dimmer can be 'abused' as a memory for screen contents this way..

dim.dim(darken_factor=64, color_filter=(0,0,0))
  Saves the current screen for later restorage and lays a filter over it -
  the default color_filter value (black) darkens the screen by blitting a 
  black surface with alpha=darken_factor over it.
  By using a different color, special effects are possible,
  darken_factor=0 just stores the screen and leaves it unchanged

dim.undim()
  restores the screen as it was visible before the last dim() call.
  If the object has been initialised with keepalive=0, this only works once.

"""</span>

import pygame

<span class="pydef">class Dimmer:</span>
    <span class="pydef">def __init__(self, keepalive=0):</span>
        self.keepalive=keepalive
        if self.keepalive:
            self.buffer=pygame.Surface(pygame.display.get_surface().get_size())
        else:
            self.buffer=None
        
    <span class="pydef">def dim(self, darken_factor=64, color_filter=(0,0,0)):</span>
        if not self.keepalive:
            self.buffer=pygame.Surface(pygame.display.get_surface().get_size())
        self.buffer.blit(pygame.display.get_surface(),(0,0))
        if darken_factor>0:
            darken=pygame.Surface(pygame.display.get_surface().get_size())
            darken.fill(color_filter)
            darken.set_alpha(darken_factor)
            <span class="pycomment"># safe old clipping rectangle...</span>
            old_clip=pygame.display.get_surface().get_clip()
            <span class="pycomment"># ..blit over entire screen...</span>
            pygame.display.get_surface().blit(darken,(0,0))
            pygame.display.flip()
            <span class="pycomment"># ... and restore clipping</span>
            pygame.display.get_surface().set_clip(old_clip)

    <span class="pydef">def undim(self):</span>
        if self.buffer:
            pygame.display.get_surface().blit(self.buffer,(0,0))
            pygame.display.flip()
            if not self.keepalive:
                self.buffer=None

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

