<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  &quot;Splurge&quot; effect module</H3>
            <p>Author: Gareth Noyce (g at korruptor.demon.co.uk)<br>
            Submission date: January 25, 2002</p>
	  
            <p>Description: Implements a horizontal or vertical pixmap flood fill.</p>
            
            <p align="center"><img src="splurge1.png" width="338" height="100"> <img src="splurge2.png" width="338" height="100"></p>

            <p>Download: <a href="splurge.zip">splurge.zip</a></p>	  

            <p>pygame version required: Any (with Surfarray)<br>
            SDL version required: Any<br>
            Python version required: Any (with Numeric)</p>

            <p>Comments: Another nicely commented demo effect script from Mr. Noyce.  This is a pretty, flexible effect, and the code is quite clear - it's a nice place to start if you're just learning to use Numeric and surfarray.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">""" splurge.py -- Gareth Noyce (aka Korruptor) v1.0 -- Testcode for SealBasher...
    
    http://www.korruptor.demon.co.uk

Splurge is another old time 2D effect, creating a nice flood fill for bitmaps. This is
especially nice when used to introduce logos, or hi-score tables, or other menu type objects
that have transparency -- when they appear over an animated backdrop. 

The algo is simplicity itself, and two directional splurges are shown in the 'splurge' function.

Vertical:
--
For each /row/ in the surface, we loop horizontally across each pixel, inserting the colour found
into each pixel below it. Some simple array slices perform this "flood copy"...

Horizontal:
--
Similar to vertical, except this time we loop for each /column/ in the surface, and using slices
copy the contents of this column into every 'column' after it. 

Viewing the effect probably explains it better than I can, and the source is trivial. The beauty
of splurges being, you don't need to erase stuff, just carry on splurging!

Extensions for Alpha controls (fading alpha on splurge given distance) and other such features
are left out of this for simplicity..."""</span>

<span class="pycomment">#-------------------------------------------------------------------------------------</span>

import pygame, pygame.image
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pycomment"># Glob decs</span>

<span class="pycomment"># Screen resolution...</span>
RES 	= array((338,100))

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def main():</span>
	<span class="pycomment">"Inisalises display, loads the image, sets up the working surf arrays, converts loaded image to an array and calls the splurge repeatedly..."</span>
     
    	<span class="pycomment"># Initialise pygame, and grab an 8bit display.</span>
    	pygame.init()

	<span class="pycomment"># setup a surface for the display</span>
    	screen_surface   = pygame.display.set_mode(RES, 0, 8)
	
	<span class="pycomment"># Load a simple image to use for the effect..</span>. 
	image_surface    = pygame.image.load("<span class="pystring">pygame_small.gif</span>")

	<span class="pycomment"># Convert this to a surf array for reference...</span>
	image_map        = pygame.surfarray.array2d(image_surface)
	
	<span class="pycomment"># Save the x/y values for later...</span>
	map_size 	 = image_surface.get_size()
	
	<span class="pycomment"># Initialise a new surfarray which we'll use as our working env...</span>
	working_map      = zeros(map_size)
	
    	<span class="pycomment"># Take the palette from the image and apply it to the screen...</span>
    	screen_surface.set_palette(image_surface.get_palette())

	while 1:
		<span class="pycomment"># Have we received an event to close the window?</span>
        	for e in pygame.event.get():
            		if e.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
                		return
		
		<span class="pycomment"># Splurge horiz and vertical until Quit...</span>
		splurge(map_size,image_map,working_map,screen_surface, 1)
		splurge(map_size,image_map,working_map,screen_surface, 0)


<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def splurge(size,imap,omap,screen,dir):</span>
	
	<span class="pycomment"># Initialise out offset vars to 0</span>
	x_cut, y_cut = 0,0

	if(dir):
		<span class="pycomment"># Go for a horizontal splurge...</span>
		for i in range(0,size[1]):
			
			for x in range(x_cut,size[0]):
				<span class="pycomment"># Set all pixels below the current x,y to the value of this x,y</span>
				omap [x][y_cut:size[1]] = imap[x][y_cut]		

			<span class="pycomment"># we've done a row, move to the next...</span>
			y_cut += 1

			<span class="pycomment"># display this frame</span>
			blit_array(screen, omap)
			pygame.display.update()

	else:
		for i in range(0,size[0]):

			for x in range(x_cut,size[0]):
				<span class="pycomment"># set all pixels in the columns (y slice) right of this x to be the same as this column (y slice)</span>
				<span class="pycomment"># This one is actually the easier slice to figure out... Start here if you're confused... </span>
				omap[x][:size[1]] = imap[x_cut][:size[1]]
		
			<span class="pycomment"># Move to next 'column'	</span>
			x_cut += 1

			<span class="pycomment"># display this frame</span>
			blit_array(screen, omap)
			pygame.display.update()

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pycomment"># ------------------------------------------------------------------------------------</span>

<span class="pycomment"># Splurge away baby...</span>
if __name__ == '<span class="pystring">__main__</span>': main()

<span class="pycomment"># End of sauce. Pass the chips...</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

