<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  'Vector Bob' Sprite/Particle Example</H3>
            <p>Author: Diez B. Roggisch (deets at web.de)<br>
            Submission date: September 17, 2002</p>
	  
            <p>Description: A well known old demo effect. A set of 3d points is transformed and every point is blitted as a surface.</p>
            <p align="center"><img src="vector_bob.jpg" width="320" height="240"></p>

            <p>Download: <a href="vector_bob.tgz">vector_bob.tgz</a></p>	  

            <p>pygame version required: Any with Surfarray<br>
            SDL version required: Any<br>
            Python version required: 2.2 with Numeric</p>

            <p>Comments: This is a vector-driven particle toolkit, where each particle is a sprite, rather than a pixel. The example script allows you to see three basic data sets - choose between them with the '1', '2' and '3' keys (on the main keyboard, not the numeric keypad.</p>
            
            <p>This code, while not very thoroughly-commented, is concise and well-structured, and just about as fast as it can be. Consider using derivitives of this code for explosion or weapons effects, or for enemy movement routines in Galaxian-style games.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
Name	:	vector bob
Desc	:

A well known old demo effect. A set of 3d points is transformed and every point is blitted as
a surface.
"""</span>

try:
	import sys, types, pygame, pygame.image, pygame.display, pygame.rect, pygame.key <span class="pycomment">#, log4py</span>
	import profile
	from Numeric import *
	<span class="pycomment">#from collision import *</span>
	
except ImportError, e:
	print "<span class="pystring">import exception occured</span>"
	print e
	sys.exit()


MODE = (640, 480)
BACKGROUND = None
BALL = None

<span class="pydef">class BobObject:</span>
	<span class="pycomment">""" A BobObject. The list _.coords are the coordinates of the points to be blitted """</span>
	<span class="pydef">def __init__(_, surface, scadjust):</span>
		<span class="pycomment">""" surface - the points are blitted as that
		scadjust - a tuple representing the screen (width/2, height/2) to center the transformed coordinates
		"""		</span>
		_.surface = surface
		_.coords = []
		_.scadjust = scadjust - array(surface.get_rect().size) / 2
		_.position = array((0,0,0))
		_.reset()
		_.colObs = []
		
	<span class="pydef">def translate(_, trans):</span>
		_.position += trans

	<span class="pydef">def reset(_):</span>
		_.orientation = array(identity(3), Float)
		
	<span class="pydef">def rotateZ(_, deg):</span>
		rMat = array(identity(3), Float)
		cos = math.cos(deg)
		sin = math.sin(deg)
		rMat[0][0] = cos
		rMat[0][1] = -sin
		rMat[1][0] = sin
		rMat[1][1] = cos
		_.orientation = matrixmultiply(_.orientation, rMat)

	<span class="pydef">def rotateY(_, deg):</span>
		rMat = array(identity(3), Float)
		cos = math.cos(deg)
		sin = math.sin(deg)
		rMat[0][0] = cos
		rMat[0][2] = -sin
		rMat[2][0] = sin
		rMat[2][2] = cos
		_.orientation = matrixmultiply(_.orientation, rMat)
		_.rot = deg

	<span class="pydef">def rotateX(_, deg):</span>
		rMat = array(identity(3), Float)
		cos = math.cos(deg)
		sin = math.sin(deg)
		rMat[1][1] = cos
		rMat[1][2] = -sin
		rMat[2][1] = sin
		rMat[2][2] = cos
		_.orientation = matrixmultiply(_.orientation, rMat)

	<span class="pydef">def screenCoords(_):</span>
		sc = []
		_.colObs = []
		for c in _.coords:
			p = matrixmultiply(c , _.orientation) + _.position
			sc.append((p[2], p[0] * 1000 / p[2] + _.scadjust[0], p[1] * 1000 / p[2] + _.scadjust[1]))

		sc.sort()
		sc.reverse()
		return map(lambda x: x[1:], sc)

	<span class="pydef">def getBobs(_):</span>
		return map(lambda x, b=_.surface: (x, b), _.screenCoords())

		

<span class="pydef">class Cube(BobObject):</span>

	<span class="pydef">def __init__(_, dim, surface, scad):</span>
		<span class="pydef">def char(a):</span>
			if a:
				return 1
			return -1
		BobObject.__init__(_, surface, scad)
		for i in xrange(8):
			_.coords.append((char(i & 1) * dim, char(i & 2) * dim, char(i & 4) * dim))

<span class="pydef">class Ring(BobObject):</span>
	<span class="pydef">def __init__(_, dim, surface, scad, segments = 8):</span>
		BobObject.__init__(_, surface, scad)
		for i in xrange(segments):
			x = dim * cos(math.pi * i / segments * 2)
			y = dim * sin(math.pi * i / segments * 2)
			_.coords.append((x, y, 0))


<span class="pydef">class Sphere(BobObject):</span>
	<span class="pydef">def __init__(_, dim, surface, scad, segments = 8):</span>
		BobObject.__init__(_, surface, scad)
		nums = [(1, 0)]
		for n in xrange(1, segments >> 1):
			deg = n / float(segments >> 1) * math.pi / 2
			radius = dim * cos(deg)
			offset = dim * sin(deg)
			num = floor(segments * cos(deg))
			if int(num) & 1:
				num += 1
			for i in xrange(num):
				x = radius * cos(math.pi * i / num * 2)
				y = radius * sin(math.pi * i / num * 2)
				_.coords.append((x, y, offset))
				_.coords.append((x, y, -offset))

		for i in xrange(segments):
			x = dim * cos(math.pi * i / segments * 2)
			y = dim * sin(math.pi * i / segments * 2)
			_.coords.append((x, y, 0))
		_.coords.append((0, 0, -dim))
		_.coords.append((0, 0, dim))
				 
<span class="pydef">class DisplayList:</span>
	<span class="pydef">def __init__(_):</span>
		_.saveStack = []
		_.bobs = []
		_.display = pygame.display.get_surface()
		
	<span class="pydef">def addBob(_, bob):</span>
		<span class="pycomment"># Create the rect in the display which has to be saved</span>
		rect = bob[1].get_rect().move(bob[0])
		bgSave = pygame.Surface(rect[2:])
		bgSave.blit(_.display, (0,0), rect)
		_.saveStack.append((bob[0], bgSave, rect))
		_.bobs.append(bob)

	<span class="pydef">def blitBobs(_):</span>
		rectList = []
		for bob in _.bobs:
			rectList.append(bob[1].get_rect().move(bob[0]))
			_.display.blit(bob[1], bob[0])
		_.bobs = []
		return rectList
	
	<span class="pydef">def restoreBackground(_):</span>
		rectList = []
		count = 0
		y = 48
		while len(_.saveStack):
			resBob = _.saveStack.pop()			
			rectList.append(resBob[2])			
			_.display.blit(resBob[1], resBob[0])
			count += 1
		return rectList




<span class="pydef">def main():</span>
	global DISPLAY
	global SCREENRECT
	global LEVEL
	global DIST_DEC
	DIST_DEC = 1
	pygame.init()
	pygame.display.init()

	rect = (0, 0) + MODE
	DISPLAY = pygame.display.set_mode(MODE, 0)
	BACKGROUND = pygame.image.load("<span class="pystring">back.png</span>").convert()
	BALL = pygame.image.load("<span class="pystring">ball.png</span>").convert()
	BALL.set_colorkey(BALL.get_at((0,0)))
	
	DISPLAY.blit(BACKGROUND, (0, 0), BACKGROUND.get_rect())

	dl = DisplayList()
	sc = array(BACKGROUND.get_rect()[2:])*0.5

	vBob = Sphere(1000, BALL, sc)
	vBob.translate(array((0, 0, 10009)))

	pygame.display.update()
	oldts = pygame.time.get_ticks() -100
	switchTs = oldts
	showBBox = 0
	speed = 50.0
	delay = 200
	at = 0
	r = 1.0
	while 1==1:
		ts = pygame.time.get_ticks()
		elapsed = ts - oldts
		oldts = ts
		pygame.event.pump()		
		keystate = pygame.key.get_pressed()
		if keystate[pygame.K_ESCAPE]:
			break

		if keystate[pygame.K_1]:
			vBob = Cube(1000 / 1.5, BALL, sc)
			vBob.translate(array((0, 0, 10009)))

		if keystate[pygame.K_2]:
			vBob = Sphere(1000, BALL, sc)
			vBob.translate(array((0, 0, 10009)))

		if keystate[pygame.K_3]:
			vBob = Ring(1000, BALL, sc)
			vBob.translate(array((0, 0, 10009)))

		invalidRects = dl.restoreBackground()

		vBob.reset()
		r = r + speed * elapsed / 1000
		vBob.rotateX((r / 180.0) * math.pi)
		vBob.rotateY((r / 180.0) * math.pi)

		bobs = vBob.getBobs()
		for b in bobs:
			dl.addBob(b)
		invalidRects += dl.blitBobs()
		pygame.display.update(invalidRects)		
		pygame.time.wait(50)

if __name__ == "<span class="pystring">__main__</span>":
	main()
	<span class="pycomment">#profile.run('main()')</span>


</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

