<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Vertical Image Wipe Effect</H3>
            <p>Author: Gareth 'Korruptor' Noyce (g at korruptor.demon.co.uk)<br>
            Submission date: February 12, 2002</p>
	  
            <p>Description: A function that will progressively 'wipe' a supplied image with a given color, using surfarray.</p>
            
            <p align="center"><img src="alpha-wipe.png" width="320" height="200"></p>

            <p>Download: <a href="vertical_alpha-wipe.zip">vertical_alpha-wipe.zip</a></p>	  

            <p>pygame version required: Any (with surfarray)<br>
            SDL version required: Any<br>
            Python version required: 2.0 or newer (with Numeric)</p>

            <p>Comments: Mr. Noyce is clearly enjoying playing with Numeric and Surfarray; I've gotten a flurry of effects submissions from him in the past month.  This function gradually 'wipes' an image from top to bottom with a solid color; alpha effects keep the transition gradual and attractive.  The code invites generalization - as Mr. Noyce mentions in the included readme file, horizontal and diagonal wipes should be easy to add.  A sample image is included, so you can see the function in action when the module is run as a script.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

import pygame, pygame.image
from pygame.surfarray import *
from pygame.locals import *

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pycomment"># Glob decs</span>

<span class="pycomment"># Screen resolution...</span>
RES 	= (320,200)

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def main():</span>
    	<span class="pycomment"># Initialise pygame, and grab an 8bit display.</span>
    	pygame.init()
    	screen_surface = pygame.display.set_mode(RES, FULLSCREEN)

	<span class="pycomment">
	# Load a dummy image</span>
    	image = pygame.image.load("<span class="pystring">pygame_small.bmp</span>")

	<span class="pycomment">
	# Give it some colourkey info...</span>
    	image.set_colorkey((255,255,255))

	<span class="pycomment">
	# Create a surface suitable for pixels_alpha()</span>
    	image_surface = pygame.Surface((image.get_size()), SRCALPHA, 32)
    
	<span class="pycomment">
	# Blit the dummy image onto it...</span>
    	image_surface.blit(image, (0,0))
    
        <span class="pycomment"># Call our "fade" module, and wait for it to do it's stuff...</span>
        do_vert_alpha_fade(image_surface, 0, 16, (255,255,0), screen_surface)

        <span class="pycomment"># Bye!</span>
        return

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def do_vert_alpha_fade(image_surface, offset, num_fade_rows, background_colour, display_surface):</span>
    <span class="pycomment">""" For a given number of rows, decrements the alpha of all pixels on the row. By contantly moving down a 
    row, we cn fade out the entire surface..."""</span>
        
    size = image_surface.get_size()

    <span class="pycomment"># Calculate the amount we will decerement each alpha value per row...</span>
    <span class="pycomment"># It helps if num_rows is a whole divisor of 256...</span>
    fade_factor = int(256/num_fade_rows)	

    <span class="pycomment"># for each row in the image_surface...</span>	
    for i in range(0,(size[1]+num_fade_rows)):

        <span class="pycomment"># Make a new array of alpha values for the surface as we deleted the last one...</span>
        arr = pygame.surfarray.pixels_alpha(image_surface)

		
        <span class="pycomment"># Increment the scanline offset to move to the next row...</span>
        offset +=1
		
        for i in range(0,num_fade_rows):

            <span class="pycomment"># Bounds check...</span>
            if((offset - i) < 0):
                    <span class="pycomment"># Ignore rows "above" screen...</span>
                    break
            if((offset - i) > size[1] - 1):
                    <span class="pycomment"># Ignore rows "below" screen, but continue as we need to process all the rows in the surface $row times...</span>
                    continue
            else:
                    <span class="pycomment"># Images with fixed alpha values will allow you to optimise this loop away:
                    # Essentially you can insert one big row of new (identical) alpha values at a time
                    # instead of iterating over each pixel, decrementing the individual alpha value for each...
                    #
                    # Unforuntately, I'm fairly sure I'll need vairable alpha ranges for my logos, so per pixel 
                    # accuracy is going to be useful. Hence the slow version! :-)</span>

                    <span class="pycomment"># for each pixel in this row...</span>
                    for j in range(0, (size[0]-1)):

                            <span class="pycomment"># Check it's current value...</span>
                            if (arr[j][offset-i] <= fade_factor):
                            
                                    <span class="pycomment"># Hmmm, number smaller than what we plan to decrement it by.
                                    # Set it to zero (to prevent 8bit-int looping and giving us a full alpha value again!)</span>
                                    arr[j][offset-i]=0
                            else: 

                                    <span class="pycomment"># Or decrement it by the fade amount...</span>
                                    arr[j][offset-i] -= fade_factor
	

        <span class="pycomment"># Right, delete the alpha_array so the surface is no longer locked...</span>
        del arr

        <span class="pycomment"># Display the newly calculated surface over the supplied background colour...</span>
        display_surface.fill(background_colour)
        display_surface.blit(image_surface, (0, 64))

   	<span class="pycomment"># Show our audience...</span>
        pygame.display.update()

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
if __name__ == '<span class="pystring">__main__</span>': main()
<span class="pycomment"># End of sauce. Pass the chips...</span>


</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

