<?php include "../global/header.html"; ?>
    
<!-- Summary begins here -->

<table bgcolor="#FFFFFF" width="80%" align="center" cellpadding="20">
    <tr>
        <td>
            <H3>Title:  Water Effects Script</H3>
            <p>Author: Pete Shinners and Gareth Noyce (korruptor at mac.com)<br>
            Submission date: May 06, 2002</p>
	  
            <p>Description: A script that demonstrates several textured water effects.</p>
            
            <p align="center"><img src="water.png" width="320" height="200"></p>

            <p>Download: <a href="pygame-water.zip">pygame-water.zip</a></p>	  

            <p>pygame version required: 1.3 (with Surfarray)<br>
            SDL version required: Any<br>
            Python version required: 2.0 (with Numeric)</p>

            <p>Comments: Another thoroughly-commented, Numeric-powered effect from Gareth Noyce.  Read the pygame-water.txt file that comes in this archive; Mr. Noyce does an excellent job of summarizing the script.  The effect itself was ported from the C original, which was written by Scott Scriven, and is available from the <a href="http://www.libsdl.org/projects/water/index.html">SDL Projects page</a>.  Featuring speed-optimized code from the creator of pygame, Pete Shinners, water.py really shows that dramatic pixel-based effects are possible using Numeric and pygame.</p>

	  <p>Note: The original 'pygame-water.zip' archive had a mis-named file that caused the script to fail on case-sensitive operating filesystems. The archive has been replaced with a fixed version.</p>
            
            <?php   $messages = explode("\n", `ls *.msg`);
                    array_pop($messages); ?>
                    
            <p><a href="#MESSAGES">Messages:</a> <?php echo count($messages); ?></p>
        </td>
    </tr>
</table>

<!-- Summary ends here -->

<!-- Code begins here -->

<pre>

<span class="pycomment">"""
Name		: Water.py
Version		: 1.0
Authors		: Pete "Shredwheat" Shinners and Gareth "Korruptor" Noyce, based on code by Scott Scriven.
Description	: (By G.N.)

    This is a Pygame port of the SDL water effect (http://www.libsdl.org/projects/water/index.html), which is 
    Scot Scriven's port of his old DOS routine (which I started the port from); based on demo code by Federico Feroldi.

    Many thanks to Pete Shinners for guiding me through the Numeric optimisations, fixing the texture mapping
    function and sorting out my mess! It'd never have run fast without his help.

    I've left my direct per-pixel code in the comments in the hope it'll help you work through the Numeric stuff
    that Pete has added. The texturing is serious voodoo though. ;-)

    Note: as this version uses 8bpp it's important that your background images use a sorted palette with a 256
    colour range. You can't just slap any old background in here unfortunately. I've used the same background
    as the original for completness. A quick hack would be for you to draw over it using the same palette and 
    apply the result. ;-) 

Keyboard Controls	:

    "w" 	== "Singing in the rain..."
    "b" 	== "Drop the kids off at the pool" -- repeatedly press b to throw another kid in the pool
    "s" 	== "Everbody gone surfin'"
    "ratbutton" == Quit
    
"""</span>

import pygame, pygame.transform, pygame.image
from operator import *
from random import *
from math import *
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pycomment"># Glob decs</span>

<span class="pycomment"># Screen resolution...</span>
RES 	= array((320,200))
CHUNKY  = RES/2
PI 	= 3.14159
DEG2RAD = PI/180

<span class="pycomment"># -----------------------------------------------------------------------------------</span>
<span class="pydef">def main():</span>
    <span class="pycomment">"Takes the user input, controls the water mode, calls the various height map mods"</span>
    
    <span class="pycomment"># Initialise pygame, and grab an 8bit display.</span>
    pygame.init()
    screen_surface = pygame.display.set_mode(RES,0, 8)

    <span class="pycomment"># Load a source image which we'll use as a background texture for the refraction</span>
    <span class="pycomment"># The palette for this is an ordered 256 colour range for simplicity...</span>
    <span class="pycomment"># The texture buffer is flattened for easier indexing in the texture_map func.</span>
    texture = pygame.image.load("water.gif")

    <span class="pycomment"># Good idea! Fit the texture to the chunky size... </span>
    texture = pygame.transform.scale(texture, CHUNKY)
    
    
    texture_buff = pygame.surfarray.array2d(texture)
    texture_buff = texture_buff.flat

    <span class="pycomment"># Two blank height maps</span>
    height_buffer = [zeros(CHUNKY),zeros(CHUNKY)]

    <span class="pycomment"># Buffer to draw on...</span>
    water_buffer = zeros(CHUNKY)

    <span class="pycomment"># Texture Lookup Table</span>
    texture_lut = make_indices_array(CHUNKY)


    <span class="pycomment"># Pygame Surface object which will take the surfarray data and be translated into a screen blit... </span>
    water_surface = pygame.Surface((CHUNKY[0], CHUNKY[1]), 0, 8)
        
    <span class="pycomment"># apply the same palette to surface</span>
    water_surface.set_palette(texture.get_palette())
    screen_surface.set_palette(texture.get_palette())

    <span class="pycomment"># Pointer to the height_buffer we're using...</span>
    hpage = 0

    <span class="pycomment"># Rain drop locators</span>
    x = 80
    y = 80

    <span class="pycomment"># initial surfer angles and placement...</span>
    xang = 0
    yang = 0
    offset = 0
    ox =CHUNKY[0]/2
    oy =CHUNKY[1]/2

    <span class="pycomment"># Water density - change this for jelly or mud effects</span>
    density = 4

    <span class="pycomment"># bobble height</span>
    pheight = 800

    <span class="pycomment"># Strength of the light - increase this for different lighting...</span>
    light = 1

    <span class="pycomment"># Size of blobs</span>
    radius = 15
    
    <span class="pycomment"># Mode 1 = rain (random)</span>
    <span class="pycomment"># Mode 2 = Surfer</span>
    <span class="pycomment"># Mode 3 = Blob</span>
    mode = 0

    <span class="pycomment"># Fruity loops...</span>
    while 1:

        pygame.event.pump()

        <span class="pycomment"># Check for keyboard input...</span>
        keyinput = pygame.key.get_pressed()

        <span class="pycomment"># If ESC or "QUIT" events occurred, exit...</span>
        if keyinput[K_ESCAPE] or pygame.event.peek(QUIT):
            return

        if keyinput[K_w]:
            <span class="pycomment"># Set a raindrop mode...</span>
            mode = 1 

        if keyinput[K_s]:
            <span class="pycomment"># Set the surfer mode...</span>
            mode = 2

        if keyinput[K_b]:
            <span class="pycomment"># Set the blob mode...</span>
            mode = 3

        if mode == 1:
            <span class="pycomment"># Make some noise!</span>
            <span class="pycomment"># pick a random position for our 'drop'</span>
            x = randrange(2,(CHUNKY[0])-2)
            y = randrange(2,(CHUNKY[1])-2)

            <span class="pycomment"># Add it to the height map we're currently working on...</span>
            height_buffer[hpage][x][y] = randrange(1,pheight<<2)

        elif mode == 2:
            <span class="pycomment"># Surfer mode...</span>

            <span class="pycomment"># Calc the new position (could slap this in a table)...</span>
            x = ((CHUNKY[0]/2)-10)*sin((xang*DEG2RAD) * 2)
            y = ((CHUNKY[1]/2)-10)*cos((yang*DEG2RAD) * 3)
            
            xang += 2
            yang += 1

            <span class="pycomment"># Draw a cross in the height map... </span>
            height_buffer[hpage][int((ox+x))][int((oy+y))] = pheight
            height_buffer[hpage][int((ox+x)+1)][int((oy+y))] = pheight >> 1
            height_buffer[hpage][int((ox+x)-1)][int((oy+y))] = pheight >> 1
            height_buffer[hpage][int((ox+x))][int((oy+y)+1)] = pheight >> 1
            height_buffer[hpage][int((ox+x))][int((oy+y)-1)] = pheight >> 1

        elif mode == 3:
            <span class="pycomment"># Blob mode...</span>
            x = randrange(2,(CHUNKY[0])-2)
            y = randrange(2,(CHUNKY[1])-2)

            <span class="pycomment"># Draw a big blob in the height map</span>
            heightBlob(x,y, pheight, radius, height_buffer[hpage])

            <span class="pycomment"># Reset the mode. Don't dive into the shallow end. </span>
            mode = 0



        <span class="pycomment"># Draw the water and smooth the map...</span>
        water_buffer = Draw_water(water_buffer, height_buffer[hpage], texture_buff, light-1, texture_lut)
        Calc_water(hpage^1, height_buffer, density)

        <span class="pycomment"># flip to the 'old' height map...</span>
        hpage ^= 1

        <span class="pycomment"># show our audience</span>
        blit_array(water_surface, water_buffer)
        temp = pygame.transform.scale(water_surface, screen_surface.get_size())
        screen_surface.blit(temp, (0,0))
        
	pygame.display.update()


<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def make_indices_array(shape):</span>
    <span class="pycomment">"creates a 3d array where each 2d index is the value"</span>
    a = indices(shape[::-1])
    return transpose(a, (0,2,1))[::-1]
    return a

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def texturemap(flattened_texture, heightmap, indices_array):</span>
    <span class="pycomment">"tie it all together (all must have same 2d dimensions)"</span>
    shape = heightmap.shape

    distortion = heightmap #>> 1

    indices = array(indices_array)
    indices[0] += distortion
    indices[1] += distortion
    indices[0] %= shape[0]
    indices[1] %= shape[1]

    lookup = indices[0]*shape[1] + indices[1]    
    mapped = take(flattened_texture, lookup.flat)
    return reshape(mapped, shape)

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def Draw_water(dest, map, texture, LightModifier, indicies):</span>
    <span class="pycomment">"Calcs the heights slopes, applies texturing, returns for screen draw"</span>

    <span class="pycomment"># For each pixel in the buffer, the delta = this_pixel - next_pixel. We don't calculate the edges...</span>
    h_map = zeros(CHUNKY)

    thispix = map[1:-1,1:-1] 
    nextpix = map[:-2,1:-1]  
    
    h_map[1:-1,1:-1] = thispix - nextpix
    
    <span class="pycomment"># The array of deltas is then used in the texture mapping to grab source pixels</span>
    <span class="pycomment"># Note: the "python" version of the texture mapping routine is in comments below.</span>
    h_map  += texturemap(texture,h_map, indicies) 

    <span class="pycomment"># Quick diversion:</span>
    <span class="pycomment">#	for no texturing, comment out the line above, to remove lighting, change the '+=' to '='</span>
            

    <span class="pycomment"># Ramp down by our lighting modifier...</span>
    h_map >>= LightModifier

    <span class="pycomment"># Make sure all values are between 0 and 255 (maps to the palette)</span>
    dest = clip(h_map,0,255)
    



    <span class="pycomment"># The slow version... (unless you run this on an SGI you lucky, lucky man...)
    # --------------------------------------------------------------------------------
    #
    #dx = 0
    #dy = 0
    #x = 0
    #y = 0
    #c = 0

    #for y in range(1, (CHUNKY[1])-1):
    # 	for x in range(1,(CHUNKY[0])-1):
    #	    Calculate the slope for the lighting...
    #        dx = map[x][y] - map[x+1][y]
    #        dy = map[x][y] - map[x][y+1]

    #        Use the slope as offsets and apply texture mapping to the lighting value
    #        c = texture[mod((dx)+x,CHUNKY[0])][mod((dy)+y,CHUNKY[1])] - (dx>>LightModifier)
    #        c = dx>>LightModifier

    #         Bounds check it...
    #        if (c < 0):
    #            c = 0
    #        elif (c > 255):
    #            c = 255
            
    #        Write this to our surface
            
    #        dest[x][y] = c
    #
    # --------------------------------------------------------------------------------



    
    # Return the buffer for screen draw...</span>
    return dest

<span class="pycomment"># ------------------------------------------------------------------------------------</span>
<span class="pydef">def Calc_water(opage, map, density):</span>
    <span class="pycomment">"Performs the smothing of the height map..."</span>
    
    <span class="pycomment"># Setup the height maps for reference. </span>
    new_page = map[opage]
    old_page = map[opage^1]

    
    center = new_page[1:-1,1:-1]
    origcenter = array(center)
    center[:] = old_page[2:,2:]
    
    center += old_page[1:-1,2:]
    center += old_page[:-2,2:]
    center += old_page[2:,1:-1]
    center += old_page[:-2,1:-1]
    center += old_page[2:,:-2]
    center += old_page[1:-1,:-2]
    center += old_page[:-2,:-2]
    center >>= 2
    center -= origcenter
    center -= (center>>density)


    <span class="pycomment"># The slow version... 
    # -------------------------------------
    #
    # for y in range(1, (CHUNKY[1])-1):
    #    for x in range(1, (CHUNKY[0])-1):

    #        newh = ((old_page[x-1][y-1] 
    #                + old_page[x][y-1] 
    #                + old_page[x+1][y-1] 
    #                + old_page[x-1][y] 
    #                + old_page[x+1][y] 
    #                + old_page[x-1][y+1] 
    #                + old_page[x][y+1] 
    #                + old_page[x+1][y+1]) >> 2) - new_page[x][y]

    #        new_page[x][y] = newh - (newh >> density)</span>
<span class="pycomment"># ------------------------------------------------------------------------------------</span>

<span class="pydef">def heightBlob(x, y, height, radius, h_map):</span>
    <span class="pycomment">"Draws a large circle in the height map - Doesn't do the sine effect of the original"</span>

    rquad = 0
    cx = 0
    cy = 0
    cyq = 0
    left = 0
    top = 0
    right = 0
    bottom = 0

    rquad = radius * radius

    <span class="pycomment"># Set the dimensions</span>
    left = -radius
    right = radius
    top = -radius
    bottom = radius

    <span class="pycomment"># Clip it's edges if our placement is going to go south</span>
    if ((x - radius) < 1):
        left -= ((x - radius) - 1)
    if ((y - radius) < 1):
        top -= ((y-radius)-1)
    if ((x + radius) > CHUNKY[0] - 1):
        right -= (x + radius - CHUNKY[0] + 1)
    if ((y + radius) > CHUNKY[1] - 1):
        bottom -= (y + radius - CHUNKY[1] + 1)

    <span class="pycomment"># This draws a large circle in the height map.
    # The original version sloped the height on the edges of the circle
    # to create a "sineblob", but this was a bit slow and didn't look much
    # better than just sticking a large blob in there instead. :-)</span>
    for cy in range (top, bottom):
        cyq = cy*cy
        for cx in range(left, right):
            if(cx*cx + cyq < rquad):
                h_map[cx+x][cy+y] += height

    
<span class="pycomment"># -------------------------------------------------------------------------------</span>
<span class="pycomment"># Brody! We're gonna need a bigger boat!</span>
if __name__ == '<span class="pystring">__main__</span>': main()

<span class="pycomment"># You wanna see a scar? I've gotta scar...</span>

</pre>

<!-- Code ends here -->

<!-- Messages begin here -->

<?php include "../global/messages.php"; ?>

<?php include "../global/footer.html"; ?>

